#! /usr/bin/env python
######################################################################
# summarize_csv_files.py
######################################################################
import os, sys, subprocess, os.path
import csv
import argparse
from utilities import *

debug = False                                   #Flag to activate prints to debug
        
# SUMMARIZE_CSV_FILE FUNCTION 

def summarize_csv_files(csv_list):
    #=========================================================================================================
    #  Get parameters
    #=========================================================================================================
    #parser = argparse.ArgumentParser()
    #List with the csv
    #parser.add_argument('csv_list', nargs='+', help="List with the names of the csv files")
    #args = parser.parse_args()
    #csv_list = args.csv_list
    
    print "List of files: %s"%csv_list
    #all_files = os.listdir(csv_directory)
    list_csv = []
    list_csv = sorted(csv_list)
    l = []
    
    for filename in list_csv:
        with open(filename, 'r') as f:
            #skips first line
            reader = csv.reader(f)
            next(reader)
            for row in reader:
                #reads proc
                l.append(int(row[0]))
 
    #Removes procs that doesn't exist in all the files to be joined
    if len(list_csv) > 1:
        procs = sorted(set([x for x in l if l.count(x) > 1]))
    else:
        procs = sorted(l)
    
    if (debug):
        print l
        print procs
    
    #Assigns a name to the final file
    measured_file     = './' + filename + '_summary_phases.csv'
    if os.path.exists(measured_file):
        os.system('rm %s'%measured_file)
        
    #Initialize variables
    min_procs= min(procs)
    items    = len(list_csv)
    first    = True
    metrics  = {}
    accum    = 0.0
    
    if (debug):
        print procs
        print items
    
    for P in procs:
        Eff_lst = []
        LB_list = []
        Ser_lst = []
        Trf_lst = []
        CEff_ls = []
        Tcpu_ls = []
        Time_ls = []
        ui_list = []
        IPC_lst = []
        
        if (debug):
            print P
            
        for filename in list_csv:
            Eff_lst.append(get_fields('Eff'    , str(P), filename))
            LB_list.append(get_fields('LB'     , str(P), filename))
            Ser_lst.append(get_fields('Ser'    , str(P), filename))
            Trf_lst.append(get_fields('Trf'    , str(P), filename))
            CEff_ls.append(get_fields('CommEff', str(P), filename))
            Tcpu_ls.append(get_fields('Tcpu'   , str(P), filename))
            Time_ls.append(get_fields('Time'   , str(P), filename))
            ui_list.append(get_fields('ui'     , str(P), filename))
            IPC_lst.append(get_fields('IPC'    , str(P), filename))
        
        #Do calculation by this number of processes
        #Efficiency = (Tcpu1*Eff1 + Tcpu2*Eff2 + ...) / (Tcpu1+Tcpu2+...)
        #LB = Efficiency/(Max1+Max2+...)
        #Ser= (sum(Max(ideal))/phases
        #Trf= ((sum(Max(real))/phases)/Ser :: Max(real) = sum(Trfi*Seri)
        
        Eff_accum  = 0.0
        Max_sum    = 0.0
        Max_ideal  = 0.0
        Max_real   = 0.0
        IPC_sum    = 0.0
        Tcpu_total = 0.0 #Sum of Tcpu/phase
        E_time_tot = 0.0 #Sum elapsed time/phase
        ui_total   = 0.0 #Sum of Useful inst/phase  
        
        #List to arrays to operate with their data
        el  = array(Eff_lst)      #Efficiency list
        lbl = array(LB_list)      #LB list
        srl = array(Ser_lst)      #Serialization
        cel = array(CEff_ls)      #CommEff (Max)
        trl = array(Trf_lst)      #Transfer
        tcl = array(Tcpu_ls)      #CPU time
        ttl = array(Time_ls)      #Elapsed time
        uil = array(ui_list)      #Useful Instructions
        ipl = array(IPC_lst)      #IPC
        
        #Looks into the map for the corresponding element (for proc = {8, 16, ...} gets {Eff, LB, etc}
        for i in range(items):
            Eff_accum   += (el[i])*(tcl[i])                        #Efficiency accumulated (Tcpu_phase_n*Eff_phase_n + ...)
            Max_sum     += ((el[i])*(ttl[i]))/lbl[i]               #LB = Eff/Max => Max = Eff/LB TcpuMAX **********Tcpu(i)
            Max_ideal   += (((el[i])*(ttl[i]))/lbl[i])/srl[i]      #Max serialization (ideal) ***********Tmax_ideal(i)
            Max_real    += cel[i]                                  #Max real for transfer
            Tcpu_total  += tcl[i]                                  #Accum CPU time
            E_time_tot  += ttl[i]                                  #Acum Elapsed time
            ui_total    += uil[i]                                  #Accum Useful instructions
            IPC_sum     += ipl[i]                                  #Accum IPC
             
        Eff_total = float(Tcpu_total/E_time_tot)                  #Efficiency all phases
        LB_total  = float(Tcpu_total/Max_sum)                     #LB all phases
        Ser_total = float(Max_sum/Max_ideal)                      #Serialization
        Comm_EFF  = float(Max_sum/E_time_tot)                     #CommEff
        Trf_total = float(Comm_EFF/Ser_total)                     #Transfer
        CEf_total = float(Max_real/items)                         #Max, CommEff (just in case)
        IPC_total = float(IPC_sum/items)                          #IPC all phases (average)
        err_total = float((Eff_total - LB_total*Ser_total*Trf_total)/Eff_total) #Error
        Est_total = float(Tcpu_total/Eff_total)                   #Estimated time
        if (first):
            T0    = Est_total
            first = False
        prd_s = float(T0/Est_total)                               #Scalability
        accum += Tcpu_total
         
        #Saves values in the dictionary
        metrics[int(P)] = Eff_total, LB_total, Ser_total, Trf_total, CEf_total, abs(err_total), float(Tcpu_total), float(E_time_tot), float(ui_total), IPC_total, Est_total, prd_s
     
    emit_list(measured_file, 'proc, Eff, LB, Ser, Trf, CommEff, err, TCPU, Time, ui, ipc, prediction, Scaled_eff\n', metrics)
    print "%s summarizes model factors from evaluated phases..."%measured_file
      
    return (measured_file)
