#! /usr/bin/env python
##############################################
# utilities
# methods used in some of the scripts
##############################################
import csv
import os.path
import subprocess
#from basicanalysis.projection_efficiency import final_params

debug = False
SCRIPTS_DIR= os.environ['BASIC_ANALYSIS_HOME']

def dict_param(indat):
    params = indat.read().splitlines()
    colu = []
    for element in params:
        if len(element) > 1:
            f = list(element.split(' '))
            aux_value = f[0] + f[1]
            colu.extend(aux_value.split(':'))
        
    i = iter(colu)
    b = dict(zip(i, i))
    if (debug):
        print colu
        print b
    return b

def effective_param(d_indat, d_com_line, name_params, confFile, dict_Default):
    final_params = {}
    
    for param in name_params:
        if (debug):
            print param
        
        # 1 - 1
        if ((confFile == True) and (d_com_line[param] != None)):
            final_params[param] = d_com_line[param]
        # 1 - 0
        elif ((confFile == True) and (d_com_line[param] == None)):
            final_params[param] = d_indat[param]
        # 0 - 1
        elif ((confFile == False) and (d_com_line[param] != None)):
            final_params[param] = d_com_line[param]
        # 0 - 0
        else:
            final_params[param] = dict_Default[param]
    
    return final_params

def do_command(command, verbose_ON):
    if (verbose_ON):
        print command
    process = subprocess.Popen(command)
    process.wait()
    if process.returncode != 0:
        print ('error', process.returncode , ' reported by command: ', 
               ' '.join(command))

def do_command_stdout(command, out, verbose_ON):
    if (verbose_ON):
        print command
    process = subprocess.Popen(command, stdout=out)
    process.wait()
    if process.returncode != 0:
        print ('error', process.returncode , ' reported by command: ', 
               ' '.join(command))
        
def do_parse_2D(stats_file, tag, col):
    with open(stats_file) as f:
        for line in f:
            words = line.split('\t')
            if len(words) > 1:
                if words[0] == tag:
                    if words[col] == '' or words[col] == '\n':
                        #print "Empty"
                        value = 0.0
                    elif words[col] == '-nan':
                        #print "NAN"
                        value = 0.0
                    else:
                        value = words[col]
                    return value

def simulate (dim_trace, bw, lat, CPUr, in_links, out_links, 
              D_trace_name, TMP_DIR, ppn, verbose_ON):
    command = [SCRIPTS_DIR + '/modules/dimemas_cfg_gen.py',  dim_trace, str(float(bw)), 
               '%.8f'%lat, str(CPUr), str(in_links), str(out_links), str(ppn) ]
    configuration_file = TMP_DIR + '/' + 'configuration.cfg'
    fd=open(configuration_file, 'w')
    do_command_stdout(command, fd, verbose_ON)
    ppn = 1
    command = ['Dimemas','-S',' 32K', '-pa ', D_trace_name, 
               '--ppn ', str(ppn), configuration_file ]
    if (verbose_ON):
        print D_trace_name
    Dimemas_file = TMP_DIR + '/' + D_trace_name[6:-4] + '.SIM_Dimemas.out'
    fd=open(Dimemas_file, 'w')
    do_command_stdout(command, fd, verbose_ON)
    
    with open(Dimemas_file) as f:
        war_count = 0
        for line in f:
            words = line.split(':')
            if words[0] == 'WARNING':
                war_count += 1  
                print 'SIMULATION: There are %d warnings, please check the .dim file'%war_count
#                 return -1.0
            elif (war_count == 0) and (words[0] == 'Execution time'):
                if (debug):
                    print  float(words[1][1:-1])
                return float(words[1][1:-1])
            elif (war_count > 0) and (words[0] == 'Execution time'):
                exec_time = float(words[1][1:-1])
                #print "TIME:%f"%exec_time
                if ((exec_time >= 10.000000000) and ((exec_time%10.000000000) == 0)):
                    print "[!] Please check final trace. It seems it might be wrong"
                    return -1.0
                     

def get_processes(paraver_file):
    with open(paraver_file) as f:
        for line in f:
            words = line.replace('(',':').replace(')',':').split(':')
            return int(words[9])

def wait(str_ng=None, prompt='Press return to show results...\n'):
    if str_ng is not None:
        print str_ng
        raw_input(prompt)

def emit_list(filename, header, list_name, verbose_ON):
    if (verbose_ON):
        print list_name
    with open(filename, "w") as w:
        w.write(header)
        for meas in sorted(list_name):
            i = 1
            for col in meas:
                if (debug):
                    print i, col
                if (i < len(meas)):
                    w.write(str(col) + ',')
                    i = i+1
                else:
                    w.write(str(col) + '\n')
                    i = i +1    
        #for key, val in sorted(list_name.iteritems()):
        #    w.write(str(key))
        #    for item in val:
        

def emit_projected(filename, header, list_name):
    with open(filename, "w") as w:
        w.write(header)
        for row in sorted(list_name):
            if (debug):
                print len(row)
            row_l = len(row)
            i = 1
            for col in row:
                if (debug):
                    print i, col
                if (i < row_l):
                    w.write(str(col) + ',')
                    i = i+1
                else:
                    w.write(str(col) + '\n')
                    i = i +1    

def get_dict (filename):                        
    #Reads the *.csv into a dictionary
    meas_csv = {}
    if (debug):
        print filename
    input_file = csv.DictReader(open(filename))
    for row in input_file:                       #Loads the .csv into memory
        meas_csv[row["proc"]] = {'Eff':float(row[" Eff"]), 'LB':float(row[" LB"]), 
                                 'Ser': row[" Ser"], 'Trf': row[" Trf"], 
                                 'CommEff':row[" CommEff"], 'err':row[" err"], 
                                 'Tcpu':row[" TCPU"], 'Time':row[" Time"], 'ui':row[" ui"],
                                 'IPC':row[" ipc"], 'Pred':row[" prediction"], 
                                 'Scal_eff':row[" Scaled_eff"], 
                                 'eff_ipc':float(row[" eff_ipc"]), 
                                 'eff_inst':float(row[" eff_inst"]) }
    return meas_csv

def get_fields (field_name, proc, filename):    
    #Reads data from an specific field and number of processes
    elem         = []
    dict_csv = get_dict(filename)
    
    if (debug):
        print field_name
        print dict_csv
        print proc
        print dict_csv[str(proc)][field_name]
        
    elem.append(float(dict_csv[str(proc)][field_name]))
    return elem

def get_elem (field_name, proc, filename):    
    dict_csv ={}
    elem = []
    dict_csv = get_dict(filename)
    
    if (debug):
        print field_name
        print dict_csv
        print proc
        print elem
        print dict_csv[str(proc)][field_name]
    return (float(dict_csv[str(proc)][field_name]))

def print_traces(it):
    try:
        while True:
            val = it.next()
            print "- %s"% str(val)
    except StopIteration:
        print "[EOF]"   