#! /usr/bin/env python
###############################################
# projection_efficiency.py
###############################################
import argparse
import csv
from modules.fitting_models_lmfit import projection_param
from modules.utilities import get_elem, emit_projected, dict_param, effective_param
from modules.genGNUPlot import genGNUPlotfileFactorsPredicted
from modules.genGNUPlot import genGNUPlotfilePrediction, genGNUPlotfileGlobalPred
from modules.summarize_csv_files import summarize_csv_files, os

debug = False                    #Flag to activate prints to debug

dict_Default = {'P_ser' : 'linear', 'fit_ser' : 'amdahl', 
                          'P_trf': 'linear', 'fit_trf': 'amdahl', 
                          'P_lb': 'linear', 'fit_lb': 'amdahl', 
                          'P_inst': 'linear', 'fit_Inst': 'amdahl',
                          'P_ipc': 'linear', 'fit_IPC': 'amdahl', }

###############################################
# MAIN
###############################################
SCRIPTS_DIR= os.environ['BASIC_ANALYSIS_HOME']

#==============================================
#  Get parameters 
#==============================================
parser = argparse.ArgumentParser()

#Receives the .csv from each phase (measurements from real traces)
parser.add_argument("-i", metavar='indat_projection.ban', nargs='?', help = "Configuration file")
parser.add_argument('-P_ser', choices = ['linear', 'cubic', 'log', 'squared'], nargs='?', 
                   help="Representation of processors for Serialization (default = linear)",
                   default=None)
parser.add_argument('-fit_ser', choices = ['pipeline', 'amdahl', 'log'], nargs='?', 
                    help="Fitting model for Serialization factor (default = amdahl)", 
                    default=None)
parser.add_argument('-P_trf', choices = ['linear', 'cubic', 'log', 'squared'], nargs='?', 
                    help="Representation of processors for Transfer", 
                    default=None)
parser.add_argument('-fit_trf', choices = ['pipeline', 'amdahl', 'log'], nargs='?', 
                    help="Fitting model for Transfer factor (default = amdahl)",
                    default=None)
parser.add_argument('-P_lb', choices = ['linear', 'cubic', 'log', 'squared'], nargs='?', 
                    help="Representation of processors for Load Balance",
                    default=None)
parser.add_argument('-fit_lb', choices = ['avg', 'min', 'amdahl', 'log'], nargs='?', 
                    help="Fitting model for Load Balance factor (default = amdahl)",
                    default=None)
parser.add_argument('-P_inst', choices = ['linear', 'cubic', 'log', 'squared'], nargs='?', 
                    help="Representation of processors for Load Balance",
                    default=None)
parser.add_argument('-fit_Inst', choices = ['pipeline', 'amdahl', 'log'], nargs='?', 
                    help="Fitting model for Efficiency of Instructions (default = amdahl)",
                    default=None)
parser.add_argument('-P_ipc', choices = ['linear', 'cubic', 'log', 'squared'], nargs='?', 
                    help="Representation of processors for Load Balance",
                    default=None)
parser.add_argument('-fit_IPC', choices = ['pipeline', 'amdahl', 'log'], nargs='?', 
                    help="Fitting model for Efficiency of IPC (default = amdahl)",
                    default=None)
parser.add_argument('-f', nargs='+', required = True, 
                    help="List with the names of the csv files")

args     = parser.parse_args()

if (args.i != None):
    conf_file = open(args.i, 'r')
    dargs = dict_param(conf_file)
    confFile = True
    if (debug):
        print dargs
else:
    print "Running without configuration file"
    print "--- using default values or inputs from command line"
    dargs = {}
    dargs ['P_ser'] = None
    dargs['fit_ser'] = None
    dargs['P_trf'] = None
    dargs['fit_trf'] = None
    dargs['P_lb'] = None
    dargs['fit_lb'] = None
    dargs['P_inst'] = None
    dargs['fit_Inst'] = None
    dargs['P_ipc'] = None
    dargs['fit_IPC'] = None
    confFile = False
    if (debug):
        print dargs

csv_list = args.f

conf_com_line = {}

conf_com_line['P_ser'] = args.P_ser
conf_com_line['fit_ser'] = args.fit_ser
conf_com_line['P_trf'] = args.P_trf
conf_com_line['fit_trf'] = args.fit_trf
conf_com_line['P_lb'] = args.P_lb
conf_com_line['fit_lb'] = args.fit_lb
conf_com_line['P_inst'] = args.P_inst
conf_com_line['fit_Inst'] = args.fit_Inst
conf_com_line['P_ipc'] = args.P_ipc
conf_com_line['fit_IPC'] = args.fit_Inst

name_params = (['P_ser', 'fit_ser', 'P_trf', 'fit_trf', 'P_lb', 'fit_lb', 
                              'P_inst', 'fit_Inst', 'P_ipc', 'fit_IPC'])
final_params = effective_param(dargs, conf_com_line, name_params, confFile, dict_Default)
p_ser = final_params['P_ser']
fit_ser = final_params['fit_ser']
p_trf = final_params['P_trf']
fit_trf = final_params['fit_trf']
p_lb = final_params['P_lb']
fit_lb = final_params['fit_lb']
p_inst = final_params['P_inst']
fit_Inst = final_params['fit_Inst']
p_ipc = final_params['P_ipc']
fit_IPC = final_params['fit_IPC']

fits = [fit_ser, fit_trf, fit_lb, fit_IPC, fit_Inst]
Ps = [p_ser, p_trf, p_lb, p_ipc, p_inst]

if (debug):
    print "FITS", fits
    print "PS", Ps
#Initial file of measurements to perform the projection
if len(csv_list) > 1:
    if (debug):
        print "More than one csv file, proceeding to summarize!"
    measured_file = summarize_csv_files(csv_list)
else:
    if (debug):
        print "Only one csv file, not summarizing"
    measured_file = csv_list[0]

gnuplot_file  = "./" + measured_file[:-4]

#Final file with the expected efficiencies
prediction_file  = "./" + measured_file[:-4]

data = []

with open(measured_file, 'r') as f:
    #skips first line
    reader = csv.reader(f)
    next(reader)
    for row in reader:
        #reads proc
        data.append(int(row[0]))

#Eliminates procs that doesn't exist in all the files to be joined 
procs = sorted(data)

if (debug):
    print data
    print procs
    
min_procs= min(procs)
Eff_meas = []
EffIPC_meas = []
EffInst_meas = []
Ser_meas = []
Trf_meas = []
LB_meas  = []
Tcpu_meas= []
Proc_list= []

for P in procs:

    Eff_meas.append  (get_elem('Eff'  , str(P), measured_file))
    EffIPC_meas.append  (get_elem('eff_ipc'  , str(P), measured_file))
    EffInst_meas.append  (get_elem('eff_inst'  , str(P), measured_file))
    LB_meas.append   (get_elem('LB'   , str(P), measured_file))
    Ser_meas.append  (get_elem('Ser'  , str(P), measured_file))
    Trf_meas.append  (get_elem('Trf'  , str(P), measured_file))
    Tcpu_meas.append (get_elem('Time' , str(P), measured_file))
    Proc_list.append(P)

d = []

#Here, the combinations for each fitting of each parameter:
                     
d = projection_param(Proc_list, Ser_meas, Trf_meas, LB_meas, 
                     Eff_meas, EffIPC_meas, EffInst_meas, Tcpu_meas, 
                     min_procs, p_ser, fit_ser, p_trf, fit_trf, p_lb, fit_lb, 
                     p_inst, fit_Inst, p_ipc, fit_IPC)

#TODO: change name of final file
gnuplot_file  +=  '.S.' + Ps[0] + '_' + fits[0] 
gnuplot_file += '.T.' + Ps[1] + '_' + fits[1]
gnuplot_file += '.L.' + Ps[2] + '_' + fits[2] 
gnuplot_file += '.IPC.' + Ps[3] + '_' + fits[3]
gnuplot_file += '.Inst.' + Ps[4] + '_' + fits[4] + '.gnuplot'

prediction_file  +=  '.S.' + Ps[0] + '_' + fit_ser 
prediction_file += '.T.' + Ps[1] + '_' + fit_trf
prediction_file += '.L.' + Ps[2] + '_' + fit_lb
prediction_file += '.IPC.' + Ps[3] + '_' + fit_IPC
prediction_file += '.Inst.' + Ps[4] + '_' + fit_Inst + '.csv'

emit_projected(prediction_file, 'P, Eff, LB, Ser, Trf, expected_Tcpu, EffInst, EffIPC, GlobalEff\n', d)

print measured_file
genGNUPlotfileFactorsPredicted(prediction_file, fits, Ps)
genGNUPlotfilePrediction(measured_file, prediction_file, fits, Ps)
genGNUPlotfileGlobalPred(measured_file, prediction_file, fits, Ps)
